@@ -1,14 +1,17 @@
 """Define V2 and V3 SimpliSafe systems."""
 import asyncio
+from dataclasses import dataclass
+from datetime import datetime
 from enum import Enum
 import logging
-from typing import Any, Callable, Coroutine, Dict, List, Set, Type, Union
+from typing import Any, Callable, Coroutine, Dict, List, Optional, Set, Type, Union
 
 from simplipy.entity import Entity, EntityTypes
 from simplipy.errors import PinError, SimplipyError
 from simplipy.lock import Lock
 from simplipy.sensor.v2 import SensorV2
 from simplipy.sensor.v3 import SensorV3
+from simplipy.util.dt import utc_from_timestamp
 from simplipy.util.string import convert_to_underscore
 
 _LOGGER: logging.Logger = logging.getLogger(__name__)
@@ -16,6 +19,8 @@ _LOGGER: logging.Logger = logging.getLogger(__name__)
 VERSION_V2 = 2
 VERSION_V3 = 3
 
+EVENT_SYSTEM_NOTIFICATION = "system_notification"
+
 CONF_DEFAULT: str = "default"
 CONF_DURESS_PIN: str = "duress"
 CONF_MASTER_PIN: str = "master"
@@ -84,6 +89,24 @@ def get_entity_class(
     return ENTITY_MAP[version].get(entity_type, ENTITY_MAP[version][CONF_DEFAULT])
 
 
+@dataclass(frozen=True)  # pylint: disable=too-many-instance-attributes
+class SystemNotification:
+    """Define a representation of a system notification."""
+
+    notification_id: str
+    text: str
+    category: str
+    code: str
+    timestamp: datetime
+
+    link: Optional[str] = None
+    link_label: Optional[str] = None
+
+    def __post_init__(self):
+        """Run post-init initialization."""
+        object.__setattr__(self, "timestamp", utc_from_timestamp(self.timestamp))
+
+
 class SystemStates(Enum):
     """States that the system can be in."""
 
@@ -122,13 +145,15 @@ class System:
         """Initialize."""
         self._get_subscription_data: Callable[..., Coroutine] = get_subscription_data
         self._location_info: dict = location_info
+        self._notifications: List[
+            SystemNotification
+        ] = self._generate_system_notification_objects()
         self._request: Callable[..., Coroutine] = request
-        self.locks: Dict[str, Lock] = {}
-        self.sensors: Dict[str, Union[SensorV2, SensorV3]] = {}
-
         self._state: SystemStates = self._coerce_state_from_string(
             location_info["system"]["alarmState"]
         )
+        self.locks: Dict[str, Lock] = {}
+        self.sensors: Dict[str, Union[SensorV2, SensorV3]] = {}
 
     @property
     def address(self) -> str:
@@ -154,6 +179,14 @@ class System:
         """
         return self._location_info["system"]["connType"]
 
+    @property
+    def notifications(self) -> List[SystemNotification]:
+        """Return the system's current messages/notifications.
+
+        :rtype: ``List[:meth:`simplipy.system.SystemNotification`]``
+        """
+        return self._notifications
+
     @property
     def serial(self) -> str:
         """Return the system's serial number.
@@ -203,6 +236,25 @@ class System:
             _LOGGER.error("Unknown system state: %s", value)
             return SystemStates.unknown
 
+    def _generate_system_notification_objects(self) -> List[SystemNotification]:
+        """Generate message objects from the message data stored in location_info."""
+        if self._location_info["system"].get("messages") is None:
+            _LOGGER.info("Notifications unavailable in plan")
+            return []
+
+        return [
+            SystemNotification(
+                raw_message["id"],
+                raw_message["text"],
+                raw_message["category"],
+                raw_message["code"],
+                raw_message["timestamp"],
+                link=raw_message["link"],
+                link_label=raw_message["linkLabel"],
+            )
+            for raw_message in self._location_info["system"]["messages"]
+        ]
+
     async def _get_entities(self, cached: bool = True) -> None:
         """Update sensors to the latest values."""
         entities: dict = await self._get_entities_payload(cached)
@@ -258,6 +310,7 @@ class System:
         )
 
         self._location_info = location_info
+        self._notifications = self._generate_system_notification_objects()
         self._state = self._coerce_state_from_string(
             location_info["system"]["alarmState"]
         )
@@ -271,21 +324,21 @@ class System:
         raise NotImplementedError()
 
     async def get_events(
-        self, from_timestamp: int = None, num_events: int = None
+        self, from_datetime: Optional[datetime] = None, num_events: Optional[int] = None
     ) -> list:
         """Get events recorded by the base station.
 
         If no parameters are provided, this will return the most recent 50 events.
 
-        :param from_timestamp: The starting timestamp (if desired)
-        :type from_timestamp: ``int``
+        :param from_datetime: The starting datetime (if desired)
+        :type from_datetime: ``datetime.datetime``
         :param num_events: The number of events to return.
         :type num_events: ``int``
         :rtype: ``list``
         """
         params: Dict[str, Any] = {}
-        if from_timestamp:
-            params["fromTimestamp"] = from_timestamp
+        if from_datetime:
+            params["fromTimestamp"] = round(from_datetime.timestamp())
         if num_events:
             params["numEvents"] = num_events
 
@@ -422,9 +475,7 @@ class System:
         result: dict
         for operation, result in zip(tasks, results):
             if isinstance(result, SimplipyError):
-                _LOGGER.error(
-                    "Error while getting latest %s values: %s", operation, result
-                )
+                raise result
 
         # We await entity updates after the task pool since including it can cause
         # HTTP 409s if that update occurs out of sequence:
